/*
 *  cnremctrl.c - Remote control server application - see man page
 *
 *  Copyright (C) 2004 Christian Nause <cn@cntec.de>
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  V. 0.1.0, March 27, 2004 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <popt.h>
#include "cnremctrl.h"


/*! Replace the character on position pos in string str with a
    space and delete the next two characters
*/
void changePercentToSpace (char* str, int pos)
{
  int i, len;

  len = strlen (str);
  str[pos] = ' ';
  for (i = pos + 1; i < len; i++) {
    str[i] = str[i+2];
  }
  str[len-2] = '\0';

  return;
}


/*! Replace every substring %20 in string str with a single space
    character.  This function is needed for commands delivered
    from xinetd.  xinetd cannot pipe server_args to cnRemCtrl that
    contains spaces.
*/
void convertSpaces (char* str)
{
  int i;

  if ((strlen (str) > 4) && (strlen (str) < CMDSTRLENGTH-1)) {
    for (i = 0; i < (strlen (str) - 2); i++) {
      if ((str[i] == '%') && (str[i+1] == '2') && (str[i+2] == '0')) {
        changePercentToSpace (str, i);
      }
    }
  }

  return;
}



/* *********************************************************************
   * MAIN                                                              *
   *********************************************************************
*/
int main (int argc, char* argv[])
{
  FILE *logfile, *tmpfile;
  char cmd[CMDMAXCOUNT][CMDSTRLENGTH];
  char logStr[CMDSTRLENGTH] = "";
  char logfilePath[CMDSTRLENGTH] = "";
  char inBuffer[CMDTCPSTRLENGTH] = "";
  char fBuffer[512] = "";
  char tmpStr[CMDTCPSTRLENGTH] = "";
  char opt;
  int  i;
  int  returnValue;
  int  syslogFlag = 0;
  int  filelogFlag = 0;
  int  testmodeFlag = 0;
  int  cmdFindFlag = 0;
  poptContext optCon;

  struct poptOption optionsTable[] = {
    { "cmd01",   '\0', POPT_ARG_STRING, 0,             CMD01,
        "command 1 with path and options", "/path/cmd" },
    { "cmd02",   '\0', POPT_ARG_STRING, 0,             CMD02,
        "command 2 with path and options", "/path/cmd" },
    { "cmd03",   '\0', POPT_ARG_STRING, 0,             CMD03,
        "command 3 with path and options", "/path/cmd" },
    { "cmd04",   '\0', POPT_ARG_STRING, 0,             CMD04,
        "command 4 with path and options", "/path/cmd" },
    { "cmd05",   '\0', POPT_ARG_STRING, 0,             CMD05,
        "command 5 with path and options", "/path/cmd" },
    { "cmd06",   '\0', POPT_ARG_STRING, 0,             CMD06,
        "command 6 with path and options", "/path/cmd" },
    { "cmd07",   '\0', POPT_ARG_STRING, 0,             CMD07,
        "command 7 with path and options", "/path/cmd" },
    { "cmd08",   '\0', POPT_ARG_STRING, 0,             CMD08,
        "command 8 with path and options", "/path/cmd" },
    { "cmd09",   '\0', POPT_ARG_STRING, 0,             CMD09,
        "command 9 with path and options", "/path/cmd" },
    { "cmd10",   '\0', POPT_ARG_STRING, 0,             CMD10,
        "command 10 with path and options", "/path/cmd" },
    { "syslog",  's',  POPT_ARG_NONE,   &syslogFlag,   0,
        "log to syslog", "" },
    { "logfile", '\0', POPT_ARG_STRING, &filelogFlag,  LOG,
        "log to file", "/path/logfile" },
    { "test",    't',  POPT_ARG_NONE,   &testmodeFlag, 0,
        "test mode, no command will be executed", "" },
    POPT_AUTOHELP { NULL, 0, 0, NULL, 0 }
  };

  
  // Initialize cmd-strings
  for (i = 0; i < CMDMAXCOUNT; i++) {
    strcpy (cmd[i], "");
  }


  // If you get a compile warning at the following line complaining about incompatible
  // pointer type of argv: This is a problem in popt.h, not in the progam here.
  optCon = poptGetContext (NULL, argc, argv, optionsTable, 0);

  // Abort if not at least two options given
  if (argc < 2) {
    poptPrintUsage (optCon, stderr, 0);
    return EXIT_FAILURE;
  }

  // Now do options processing
  while ((opt = poptGetNextOpt (optCon)) >= 0) {
    if ((opt >= CMD01) && (opt <= CMD10)) {
      strncpy (cmd[opt - 1], poptGetOptArg (optCon), CMDSTRLENGTH - 1);
      convertSpaces (cmd[opt - 1]);
    }
    else if (opt == LOG) {
      strncpy (logfilePath, poptGetOptArg (optCon), CMDSTRLENGTH-1);
    }
  }

  // Abort, if command line processing with popt gives error
  if (opt < -1) {
    fprintf (stderr, "Error %s: %s\n",
      poptBadOption (optCon, POPT_BADOPTION_NOALIAS), poptStrerror (opt));
    return EXIT_FAILURE;
  }

  poptFreeContext (optCon);

  
  // Receive data: cnRemCtrl is called from xinetd. xinetd opens socket
  // to cnRemCtrl-WinClient, receives its tcp-stream and sends the data via
  // pipe to this server application cnRemCtrl.
  fgets (inBuffer, CMDTCPSTRLENGTH, stdin);
  inBuffer[CMDTCPSTRLENGTH - 1]='\0';


  // Systemcalls
  for (i = 0; i < CMDMAXCOUNT; i++) {
    sprintf (tmpStr, "Button%02d", i + 1);
    if (strcmp (inBuffer, tmpStr) == 0) {
      cmdFindFlag = 1;
      if (strcmp (cmd[i], "") == 0) {
        sprintf (logStr, "System command %d not defined!\n", i + 1);
      }
      else {
        if (!testmodeFlag) {
          system (cmd[i]);
          sprintf (logStr, "Command \"%s\" executed.\n", cmd[i]);
        }
        else {
          sprintf (logStr, "Testmode: System command \"%s\" not executed\n", cmd[i]);
        }
      }
      break;
    }
  }

  if (cmdFindFlag == 0) {
    sprintf (logStr, "Error: Unknown command \"%s\"\n", inBuffer);
  }

  // Print logStr to stdout. This line is only displayed in cnRemoteControl-log,
  // if the cmd doesn't write anything to stdout
  // Doesn't work proper. At some commands the line is printed behind the cmd-
  // output, at some command the line is printed at the next command.
  // printf (logStr);
  printf ("\n");

  // Logging to syslog and/or logfile if desired
  if (syslogFlag) syslog (LOG_INFO, logStr);
  if (filelogFlag) {
    logfile = fopen (logfilePath, "a");
    fprintf (logfile, logStr);
    fclose (logfile);
  }


  return EXIT_SUCCESS;
}

