/*
 *  cnRemoteControl - Remote control client application - see documentation
 *
 *  cnRemoteControl.c
 *
 *  Copyright (C) March 28, 2004 Christian Nause <cn@cntec.de>
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 */

#include <windows.h>
#include <stdio.h>
#include "cnRemoteControl.h"
#include "parfile.h"
#include "resource.h"
#include "commControl.h"


/*
********************************************************************************
* GLOBALS                                                                      *
********************************************************************************
*/
HINSTANCE hInst;
HWND      ghWnd;
SOCKET    sock;

// Struktur fuer Programm-Parameter, die unter 'File - Options' eingestellt
// werden und in inifile gespeichert werden
// Elemente: Name-String, Nummer, Typ und Default-Wert
parmStructT parmStruct[PARMCOUNT] = {
  { "server-ip",    SERVER_IP_NO,   IP4V, "NA", 0 },
  { "server-name",  SERVER_NAME_NO, STRV, "NA", 0 },
  { "server-port",  SERVER_PORT_NO, PORV, "NA", 0 },
  { "timeout",      TIMEOUT_NO,     INTV, "NA", 0 },  
  { "cmdcount",     CMDCOUNT_NO,    INTV, "NA", 0 },
  { "cmd1-active",  CMD1_ACT_NO,    BOLV, "NA", 0 },
  { "cmd1",         CMD1_NO,        STRV, "NA", 0 },
  { "cmd1-check",   CMD1_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd2-active",  CMD2_ACT_NO,    BOLV, "NA", 0 },
  { "cmd2",         CMD2_NO,        STRV, "NA", 0 },
  { "cmd2-check",   CMD2_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd3-active",  CMD3_ACT_NO,    BOLV, "NA", 0 },
  { "cmd3",         CMD3_NO,        STRV, "NA", 0 },
  { "cmd3-check",   CMD3_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd4-active",  CMD4_ACT_NO,    BOLV, "NA", 0 },
  { "cmd4",         CMD4_NO,        STRV, "NA", 0 },
  { "cmd4-check",   CMD4_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd5-active",  CMD5_ACT_NO,    BOLV, "NA", 0 },
  { "cmd5",         CMD5_NO,        STRV, "NA", 0 },
  { "cmd5-check",   CMD5_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd6-active",  CMD6_ACT_NO,    BOLV, "NA", 0 },
  { "cmd6",         CMD6_NO,        STRV, "NA", 0 },
  { "cmd6-check",   CMD6_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd7-active",  CMD7_ACT_NO,    BOLV, "NA", 0 },
  { "cmd7",         CMD7_NO,        STRV, "NA", 0 },
  { "cmd7-check",   CMD7_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd8-active",  CMD8_ACT_NO,    BOLV, "NA", 0 },
  { "cmd8",         CMD8_NO,        STRV, "NA", 0 },
  { "cmd8-check",   CMD8_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd9-active",  CMD9_ACT_NO,    BOLV, "NA", 0 },
  { "cmd9",         CMD9_NO,        STRV, "NA", 0 },
  { "cmd9-check",   CMD9_CHECK_NO,  BOLV, "NA", 0 },
  { "cmd10-active", CMD10_ACT_NO,   BOLV, "NA", 0 },
  { "cmd10",        CMD10_NO,       STRV, "NA", 0 },
  { "cmd10-check",  CMD10_CHECK_NO, BOLV, "NA", 0 },
  { "language",     LANGUAGE_NO,    STRV, "NA", 0 },
};


/*
********************************************************************************
* FUNCTIONS                                                                    *
********************************************************************************
*/
/*! Message handler options dialog
*/
LRESULT CALLBACK OptionsMsgHandler (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  char titelStr[2*CHARSPERSTR];
  char str[CHARSPERSTR] = "";
  char ipStr[CHARSPERSTR] = "";
  int  timeout, i;
  struct in_addr temp_addr;
  extern HWND ghWnd;
  

	switch (message) {
	  case WM_INITDIALOG:
	    // Eingabefelder mit Values aus parmStruct belegen
	    if (strcmp (parmStruct[SERVER_NAME_NO].value, "") == 0) {
	      // Wenn String Server-Name leer ist, pruefen, ob String IP-Adresse auch leer ist
  	    if (strcmp (parmStruct[SERVER_IP_NO].value, "") == 0) {
          // IP-Adresse auch leer, leeren String eintragen
          SetDlgItemText (hDlg, IDC_IP_OR_NAME, "");
        }
        else {
          // IP-Adresse eintragen
          SetDlgItemText (hDlg, IDC_IP_OR_NAME, parmStruct[SERVER_IP_NO].value);
        }
      }
      else {
	      // Server-Name eintragen
        SetDlgItemText (hDlg, IDC_IP_OR_NAME, parmStruct[SERVER_NAME_NO].value);
      }
      SetDlgItemText (hDlg, IDC_PORT, parmStruct[SERVER_PORT_NO].value);
      SetDlgItemText (hDlg, IDC_TIMEOUT, parmStruct[TIMEOUT_NO].value);
      
      for (i = 0; i < BUTTONCOUNT; i++) {
        // Buttontexte in Eingabefelder eintragen
        SetDlgItemText (hDlg, ID_BUTTONTEXT1 + i, parmStruct[CMD1_NO + i * 3].value);

        // Checkboxen anwaehlen
        if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE)
          SendMessage (GetDlgItem (hDlg, IDC_CHECK_SEC1 + i), BM_SETCHECK, BST_CHECKED, 0);
          
        if (parmStruct[CMD1_ACT_NO + i * 3].valueInt == TRUE)
          SendMessage (GetDlgItem (hDlg, IDC_CHECK_ACT1 + i), BM_SETCHECK, BST_CHECKED, 0);

        // Button-Text-Eingabefelder auf max. CHARSPERBUTTONTEXT Zeichen begrenzen
        SendMessage (GetDlgItem (hDlg, ID_BUTTONTEXT1 + i), EM_LIMITTEXT, (WPARAM) CHARSPERBUTTONTEXT, 0);
      }
		  return TRUE;

	  case WM_COMMAND:
	    // OK-Button
		  if (LOWORD (wParam) == IDOK) {
        // Kombifeld IP-Adresse/Server-Name einlesen und checken
        GetDlgItemText (hDlg, IDC_IP_OR_NAME, str, CHARSPERSTR);

        // Checken, falls String nicht leer
        if (strcmp (str, "") != 0) {
          // String ist gueltige IP
          if (inet_addr (str) != INADDR_NONE) {
            // Umwandlung String in numerische IP und anschliessend
            // wieder zurueck in String.
            // So werden auch gueltige Adressen ausgewertet, die nicht in der
            // dotted-Schreibweise vorliegen
            temp_addr.s_addr = inet_addr (str);
            strcpy (parmStruct[SERVER_IP_NO].value, inet_ntoa (temp_addr));
            strcpy (parmStruct[SERVER_NAME_NO].value, "");
          }
          // String wird als Hostname interpretiert
          else {
            // Keine gueltige IP. Versuchen den String als Server-Name in IP aufzuloesen.
            if (checkServer (hDlg, ipStr, str) != OK) {
              // Das ist fehlgeschlagen, checkServer hat entsprechende Messagebox gezeigt
              SetFocus (GetDlgItem (hDlg, IDC_IP_OR_NAME));
              return FALSE;
            }
            // Namensaufloesung erfolgreich, Werte in parmStruct eintragen
            strcpy (parmStruct[SERVER_IP_NO].value, ipStr);          
            strcpy (parmStruct[SERVER_NAME_NO].value, str);            
          }
        }
        // String ist leer. So in parmStruct eintragen
        else strcpy (parmStruct[SERVER_IP_NO].value, "");          


        // Umrahmungstext Buttons schreiben
        sprintf (titelStr, "Command on server %s...", parmStruct[SERVER_IP_NO].value);
        SendMessage (GetDlgItem (ghWnd, IDC_STATIC_BUTTONS), WM_SETTEXT, 0, (LPARAM) titelStr);

        
        // Feld Port-Nummer einlesen und checken
        GetDlgItemText (hDlg, IDC_PORT, str, 6);
        // Integer-Check
        if (ascIntCheck (str) == OK) {
          // Integer im zulaessigen Port-Bereich?
          if (ascPortCheck (str) == OK) {
            strcpy (parmStruct[SERVER_PORT_NO].value, str);
          }
          else {
            MessageBox (hDlg, "Invalid Port-Number!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
            SetFocus (GetDlgItem (hDlg, IDC_PORT));
            return FALSE;
          }
        }
        else {
          MessageBox (hDlg, "Invalid Port-Number!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
          SetFocus (GetDlgItem (hDlg, IDC_PORT));
          return FALSE;
        }
        
        // Feld Timeout einlesen und checken...
        GetDlgItemText (hDlg, IDC_TIMEOUT, str, 6);
        timeout = atoi (str);
        if (timeout < TIMEOUT_MIN || timeout > TIMEOUT_MAX) {
          MessageBox (hDlg, "Invalid Timeout value!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
          SetFocus (GetDlgItem (hDlg, IDC_TIMEOUT));
          return FALSE;
        }
        else {
          strcpy (parmStruct[TIMEOUT_NO].value, str);
          parmStruct[TIMEOUT_NO].valueInt = timeout;
        }
        
        // Felder Buttonbeschriftungen einlesen (CHARSPERBUTTONTEXT Zeichen moeglich)
        for (i = 0; i < BUTTONCOUNT; i++) {
          GetDlgItemText (hDlg, ID_BUTTONTEXT1 + i, str, CHARSPERBUTTONTEXT + 1);
          if (ascStringCheck (str) == FALSE) {
            MessageBox (hDlg, "Invalid Buttontext!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
            SetFocus (GetDlgItem (hDlg, ID_BUTTONTEXT1 + i));
            return FALSE;
          }
          else {
            (strcpy (parmStruct[CMD1_NO + i * 3].value, str));
            SendMessage (GetDlgItem (ghWnd, ID_BUTTON1 + i), WM_SETTEXT, 0, (LPARAM) parmStruct[CMD1_NO + i * 3].value);
          }
          
          // Sicherheitsabfrage-Checkboxen einlesen
          if (SendMessage (GetDlgItem (hDlg, IDC_CHECK_SEC1 + i), BM_GETCHECK, 0, 0) == BST_CHECKED) {
            strcpy (parmStruct[CMD1_CHECK_NO + i * 3].value, "1");
            parmStruct[CMD1_CHECK_NO + i * 3].valueInt = TRUE;
          }
          else {
            strcpy (parmStruct[CMD1_CHECK_NO + i * 3].value, "0");
            parmStruct[CMD1_CHECK_NO + i * 3].valueInt = FALSE;        
          }

          // Active-Checkboxen einlesen
          if (SendMessage (GetDlgItem (hDlg, IDC_CHECK_ACT1 + i), BM_GETCHECK, 0, 0) == BST_CHECKED) {
            strcpy (parmStruct[CMD1_ACT_NO + i * 3].value, "1");
            parmStruct[CMD1_ACT_NO + i * 3].valueInt = TRUE;
          }
          else {
            strcpy (parmStruct[CMD1_ACT_NO + i * 3].value, "0");
            parmStruct[CMD1_ACT_NO + i * 3].valueInt = FALSE;        
          }
        }        
                        
        // Eingegebene Werte abspeichern in inifile
        if (writeInifile () == MY_ERROR) {
          MessageBox (hDlg, "Cannot write ini-file!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
        }
       
			  EndDialog (hDlg, LOWORD (wParam));
			  return TRUE;
		  }
		  // Cancel button
		  if (LOWORD (wParam) == IDCANCEL) {
  
			  EndDialog (hDlg, LOWORD (wParam));
			  return TRUE;
		  }
		  break;
  }
	return FALSE;
}


/*! Initialisierung. Wird beim Programmstart vor dem Zeichnen des Hauptfensters
    (Dialogbox) aufgerufen. Dient hier zur Positionierung des Fensters.
*/
void init (HWND hWnd)
{
  char str [CHARSPERMESSAGESTR] = "";
  int i;
  int  dlgboxXSize, dlgboxYSize;
  int  screenXSize, screenYSize;
  RECT dlgboxCoord;
  extern int gWSAStartupFlag;


  // Winsock-dll freigeben
  WSACleanup ();
  gWSAStartupFlag = FALSE;
    
  // Dialogbox-Koordinaten ermitteln und in Struktur dlgboxCoord ablegen
  GetWindowRect (hWnd, &dlgboxCoord); 

  // Dialogbox-Groesse berechnen
  dlgboxXSize = dlgboxCoord.right - dlgboxCoord.left;
  dlgboxYSize = dlgboxCoord.bottom - dlgboxCoord.top;
  
  // Bildschirmgroesse ermitteln
  screenXSize = GetSystemMetrics (SM_CXSCREEN);
  screenYSize = GetSystemMetrics (SM_CYSCREEN);

  // Dialogbox an gewuenschter Position zeichnen (Abstand von
  // linker unterer Ecke 10 % der Screen-Abmessungen)
	SetWindowPos (hWnd, HWND_TOP, (int) (0.1 * screenXSize), 
	              ((int) (0.9 * screenYSize)) - dlgboxYSize, 0, 0, SWP_NOSIZE);
	 
	// ini-Datei checken und lesen
  // Datei enthaelt alle Einstellungen, die im Dialog Options einstellbar sind
  // Falls Fehler erkannt werden, wird als erstes Messagebox mit Fehler und dann das
  // Hauptfenster gezeigt
  checkInifile (hWnd);
  
  // Buttons mit gelesenen Werten beschriften
  for (i = 0; i < BUTTONCOUNT; i++) {
    SendMessage (GetDlgItem (hWnd, ID_BUTTON1 + i), WM_SETTEXT, 0, (LPARAM) parmStruct[CMD1_NO + i * 3].value);
    
    // Button nur zeichnen, wenn Flag CMDn_ACT gesetzt ist
    if (parmStruct[CMD1_ACT_NO + i * 3].valueInt == TRUE)
      EnableWindow (GetDlgItem (hWnd, ID_BUTTON1 + i), TRUE);
    else
      EnableWindow (GetDlgItem (hWnd, ID_BUTTON1 + i), FALSE);
  }

  sprintf (str, "Command on server %s...", parmStruct[SERVER_IP_NO].value);
  SendMessage (GetDlgItem (hWnd, IDC_STATIC_BUTTONS), WM_SETTEXT, 0, (LPARAM) str);
  
  // Focus auf Button 1 setzen
  SetFocus (GetDlgItem (hWnd, ID_BUTTON1));
  
  // Fenster jetzt zeichnen
  ShowWindow (hWnd, SW_SHOW);
	UpdateWindow (hWnd);

	return; 
}


/*! Standard-Aufruf Window-Prozedur
*/
LRESULT CALLBACK WndProc (HWND hWnd, UINT uiMessage, WPARAM wParam, LPARAM lParam)
{
  char tempStr [TCPRECVBUFFERLEN];
  static char msgStr [TCPSENDBUFFERLEN + TCPRECVBUFFERLEN + 50];
  char  recvStr [TCPRECVBUFFERLEN] = "";
  char  sendStr [TCPSENDBUFFERLEN] = "";
  int   errNo, i;
  WORD  wEvent, wError;
  int   byteCntSend = 0;
  int   byteCntRecv = 0;
  int   timeout = 3000;
  extern SOCKET sock;
  static int commandNo;
  extern int gWSAStartupFlag;
  static int recvDataFlag;
  HFONT logFont;
  

  switch (uiMessage) {
    // Buttons initialisieren
    case WM_SETFOCUS:      
      // Buttons zeichnen, deren Flag CMDn_ACT gesetzt ist
      for (i = 0; i < BUTTONCOUNT; i++) {
        if (parmStruct[CMD1_ACT_NO + i * 3].valueInt == TRUE)
          EnableWindow (GetDlgItem (hWnd, ID_BUTTON1 + i), TRUE);
        else
          EnableWindow (GetDlgItem (hWnd, ID_BUTTON1 + i), FALSE);
      }    

      // Fokus auf ersten aktiven Button setzen
      for (i = 0; i < BUTTONCOUNT; i++) {
        if (parmStruct[CMD1_ACT_NO + i * 3].valueInt == TRUE) {
          SetFocus (GetDlgItem (hWnd, ID_BUTTON1 + i));
          break;
        }
      }

      return 0;

    // F1-Taste gedrueckt, Hilfe aufrufen
    case WM_HELP:
      if (strcmp (parmStruct[LANGUAGE_NO].value, "german") == 0)
        sprintf (tempStr, "%s\\%s", HELPFILEPATH, HELPFILENAME_GERMAN);
      else 
        sprintf (tempStr, "%s\\%s", HELPFILEPATH, HELPFILENAME_ENGLISH);
      if (ShellExecute (NULL, "open", tempStr, NULL, NULL, SW_SHOWNORMAL) <= (HINSTANCE) 32) {
        errMessageBox (hWnd, ERR_HELPFILE, NULL, NULL);
      }
      return 0;

    // Timeout zum Empfangen der Serverantwort abgelaufen
    case WM_TIMER:
      KillTimer (hWnd, ID_TIMER);
      errMessageBox (hWnd, ERR_TIMEOUT, NULL, NULL);
      // hier kein break oder return, damit anschliessend direkt WM_SOCKET_CLOSE aufgerufen wird
         
    case WM_SOCKET_CLOSE:
      closesocket (sock);
      sock = 0;
      WSACleanup ();
      gWSAStartupFlag = FALSE;

      // Mauszeiger -> Standardzeiger
      SetCursor (LoadCursor (NULL, IDC_ARROW));
      return 0;
            
    case WM_SOCKET_NOTIFY:
      wEvent = WSAGETSELECTEVENT (lParam);   // WSA-Event ermitteln
      wError = WSAGETSELECTERROR (lParam);   // WSA-Error ermitteln

      // Behandlung der fuer WSAAsyncSelect spezifizierten Ereignisse
      // FD_CONNECT und FD_READ
      switch (wEvent) {
        case FD_CONNECT: // Rueckmeldung von connect ()
          if (wError) {
            // Mauszeiger -> Standardzeiger
            SetCursor (LoadCursor (NULL, IDC_ARROW));
            errMessageBox (hWnd, ERR_CONNECT2, parmStruct[SERVER_IP_NO].value, NULL);
            gWSAStartupFlag = FALSE;
            return 0;
          }
          
          // Kommandostring in Sendebuffer kopieren
          switch (commandNo) {
            case BUTTON1:
              strcpy (tempStr, parmStruct[CMD1_NO].value);
              strcpy (sendStr, CMD01SENDSTRING);
              break;
            case BUTTON2:
              strcpy (tempStr, parmStruct[CMD2_NO].value);
              strcpy (sendStr, CMD02SENDSTRING);
              break;
            case BUTTON3:
              strcpy (tempStr, parmStruct[CMD3_NO].value);
              strcpy (sendStr, CMD03SENDSTRING);
              break;
            case BUTTON4:
              strcpy (tempStr, parmStruct[CMD4_NO].value);
              strcpy (sendStr, CMD04SENDSTRING);
              break;
            case BUTTON5:
              strcpy (tempStr, parmStruct[CMD5_NO].value);
              strcpy (sendStr, CMD05SENDSTRING);
              break;
            case BUTTON6:
              strcpy (tempStr, parmStruct[CMD6_NO].value);
              strcpy (sendStr, CMD06SENDSTRING);
              break;
            case BUTTON7:
              strcpy (tempStr, parmStruct[CMD7_NO].value);
              strcpy (sendStr, CMD07SENDSTRING);
              break;
            case BUTTON8:
              strcpy (tempStr, parmStruct[CMD8_NO].value);
              strcpy (sendStr, CMD08SENDSTRING);
              break;
            case BUTTON9:
              strcpy (tempStr, parmStruct[CMD9_NO].value);
              strcpy (sendStr, CMD09SENDSTRING);
              break;
            case BUTTON10:
              strcpy (tempStr, parmStruct[CMD10_NO].value);
              strcpy (sendStr, CMD10SENDSTRING);
              break;
            default:
              errMessageBox (hWnd, ERR_CMD, parmStruct[SERVER_IP_NO].value, NULL);
              return 0;
          }

          // Log-Text erstellen
          removeAmpersand (tempStr);
          sprintf (msgStr, "Sending command \"%s\"...\r\n", tempStr);
          
          // Font fuer Log-Text einstellen
          logFont = CreateFont (-12, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, OUT_TT_PRECIS, 0, ANTIALIASED_QUALITY, FIXED_PITCH, "Courier New");
	        SendMessage(GetDlgItem (hWnd, IDC_LOG), WM_SETFONT, (WPARAM) logFont, 0L);          

          // Log-Text anzeigen
          SendMessage (GetDlgItem (hWnd, IDC_LOG), WM_SETTEXT, 0, (LPARAM) msgStr);

          // Kommando per TCP/IP zum Server senden
          byteCntSend = send (sock, sendStr, (int) strlen (sendStr), 0);
          if (byteCntSend < 0) {
            errMessageBox (hWnd, ERR_SEND, parmStruct[SERVER_IP_NO].value, NULL);
            return 0;
          }      

          // Aktiviere nichtblockierenden Datenempfang. Der Aufruf generiert einen
          // WSAEWOULDBLOCK-Fehler und bei Datenempfang ein FD_READ-Ereignis.
          // WSAEWOULDBLOCK-Fehler ignorieren.
          if (SOCKET_ERROR == recv (sock, recvStr, TCPRECVBUFFERLEN, 0)) {
            errNo = WSAGetLastError ();
            if (errNo != WSAEWOULDBLOCK) {
              // Alle Fehler ausser WSAEWOULDBLOCK anzeigen
              sprintf (tempStr, "Socket error %d on receive data from server (1)", errNo);
              MessageBox (hWnd, tempStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
            }
          }
                    
          // Flag ruecksetzen, das Datenempfang signalisiert
          recvDataFlag = FALSE;
         
          // Timeout setzen. Wartezeit, in der Server-Antwort eintreffen muss, bevor
          // Fehlermeldung ausgegeben wird. Bei Ablauf wird WM_TIMER ausgeloest.
          SetTimer (hWnd, ID_TIMER, parmStruct[TIMEOUT_NO].valueInt, NULL);

          return 0;


        // Daten wurden empfangen
        case FD_READ:
          KillTimer (hWnd, ID_TIMER); // Timer loeschen, damit Timeout unterbleibt

          if (wError) {
            errMessageBox (hWnd, ERR_RECEIVE, parmStruct[SERVER_IP_NO].value, NULL);
            return 0;
          }

          // Auslesen des Empfangspuffers
          byteCntRecv = recv (sock, recvStr, TCPRECVBUFFERLEN, 0);

          // Fehler beim Datenempfang
          if (byteCntRecv == -1) {
            sprintf (tempStr, "Socket error %d on recveive data from server(2)", WSAGetLastError ());
            MessageBox (hWnd, tempStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
            return 0;
          }
          // Empfangspuffer ist leer, Nachricht WM_SOCKET_CLOSE versenden
          else if (byteCntRecv == 0) {
            SendMessage (hWnd, WM_SOCKET_CLOSE, 0, 0);
            return 0;
          }
          // Empfangene Daten passen in Empfangspuffer, Endezeichen \0 ans
          // Ende der Daten schreiben
          else if (byteCntRecv < TCPRECVBUFFERLEN) {
            recvStr[byteCntRecv] = '\0';
          }
          // Empfangene Daten passen nicht in Empfangspuffer,
          // Daten abschneiden und Endezeichen \0 ans Pufferende schreiben
          else {
            recvStr[TCPRECVBUFFERLEN - 1] = '\0';
          }

          // Falls FD_READ das erste mal eintrifft, Daten in log-
          // Control ausgeben. Bei mehrfachem Empfang werden die Daten zwar
          // aus dem Empfangspuffer geholt, aber verworfen und nicht angezeigt.
          if (recvDataFlag == FALSE) {
            filterString (tempStr, recvStr, TCPRECVBUFFERLEN);
            strcat (msgStr, tempStr);
            SendMessage (GetDlgItem (hWnd, IDC_LOG), WM_SETTEXT, 0, (LPARAM) msgStr);
            // Mauszeiger -> Standardzeiger
            SetCursor (LoadCursor (NULL, IDC_ARROW));
            recvDataFlag = TRUE;
          }
          return 0;       
      }
      return 0; // switch (wEvent)
    return 0; // case WM_SOCKET_NOTIFY

    case WM_COMMAND: 
      // Check menu and buttons
      if (HIWORD (wParam) == 0) {
        switch (LOWORD (wParam)) {             
          case IDC_INIT:
            init (hWnd);
            break;
                  
          case ID_MENU_OPTIONS:
            ghWnd = hWnd;
      			DialogBox (hInst, (LPCTSTR) ID_OPTIONSDLG, hWnd, (DLGPROC) OptionsMsgHandler);
            break;
            
          case ID_MENU_EXIT:
            PostQuitMessage (0);
            break;
            
          case ID_MENU_HELP:
            if (strcmp (parmStruct[LANGUAGE_NO].value, "german") == 0)
              sprintf (tempStr, "%s\\%s", HELPFILEPATH, HELPFILENAME_GERMAN);
            else 
              sprintf (tempStr, "%s\\%s", HELPFILEPATH, HELPFILENAME_ENGLISH);
            if (ShellExecute (NULL, "open", tempStr, NULL, NULL, SW_SHOWNORMAL) <= (HINSTANCE) 32) {
              errMessageBox (hWnd, ERR_HELPFILE, NULL, NULL);
            }
            break;
            
          case ID_MENU_ABOUT:
      			sprintf (msgStr, "cnRemoteControl %s\n Christian Nause %s\n", BUILD_VERSION, BUILD_DATE);
      			strcat (msgStr, "Free Software (GNU General Public License 2)\nhttps://sourceforge.net/projects/cnremctrl");
	          MessageBox (hWnd, msgStr, "cnRemoteControl - About", MB_OK | MB_ICONINFORMATION);
            break;

          case ID_BUTTON1:
            i = 0;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON2:
            i = 1;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON3:
            i = 2;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON4:
            i = 3;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON5:
            i = 4;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON6:
            i = 5;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON7:
            i = 6;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON8:
            i = 7;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON9:
            i = 8;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          case ID_BUTTON10:
            i = 9;            
            commandNo = BUTTON1 + i;
            // Falls konfiguriert, Sicherheitsabfrage durchfuehren
            if (parmStruct[CMD1_CHECK_NO + i * 3].valueInt == TRUE) {
              sprintf (msgStr, "Execute command \"%s\"?", parmStruct[CMD1_NO + i * 3].value);
              removeAmpersand (msgStr);
  	          if (MessageBox (hWnd, msgStr, "cnRemoteControl - Check", MB_OKCANCEL | MB_ICONQUESTION) != IDOK) break;
            }
            // Mauszeiger -> Sanduhr
            SetCursor (LoadCursor (NULL, IDC_WAIT));
            errNo = openSocket (atoi (parmStruct[SERVER_PORT_NO].value), parmStruct[SERVER_IP_NO].value, hWnd);
            if (errNo != OK) {
              errMessageBox (hWnd, errNo, parmStruct[SERVER_IP_NO].value, NULL);
            }
            break;

          default: break;
        } // switch
      }   // if
      
      return 0;
      
    case WM_DESTROY:
      PostQuitMessage (0);
      return 0;

    default:
      return DefWindowProc (hWnd, uiMessage, wParam, lParam);
  } // switch (uiMessage)
}


/*! Schreibt ini-Datei mit Werten aus Parameter-Struktur
*/
int writeInifile (void)
{
  int   i;
  FILE* inifile;
  char  filename[CHARSPERSTR] = INIFILENAME;
  char  tmpStr[CHARSPERSTR] = ""; 
  extern parmStructT parmStruct[PARMCOUNT];

  
  inifile = fopen (filename, "w");
  if (inifile == NULL) return MY_ERROR;

  // File schreiben
  fprintf (inifile, "# cnRemoteControl.ini\n\n");
  for (i = 0; i < PARMCOUNT; i++) {
    // Falls Value leer, mit "" belegen
    if (strcmp (parmStruct[i].value, "") == 0) strcpy (tmpStr, "\"\"");
    else strcpy (tmpStr, parmStruct[i].value);
    fprintf (inifile, "%12s = %s\n", parmStruct[i].name, tmpStr);
  }

  fclose (inifile);

  return OK;
}


/*! Schreibt ini-Datei mit Default-Werten
*/
int writeDefaultInifile (void)
{
  FILE* inifile;
  char filename[CHARSPERSTR] = INIFILENAME;
  extern parmStructT parmStruct[PARMCOUNT];

  
  inifile = fopen (filename, "w");

  if (inifile == NULL) return MY_ERROR;

  fprintf (inifile, "# cnRemoteControl.ini\n\n");
  fprintf (inifile, "%12s = %s\n", parmStruct[SERVER_IP_NO].name, SERVER_IP_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[SERVER_NAME_NO].name, SERVER_NAME_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[SERVER_PORT_NO].name, SERVER_PORT_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[TIMEOUT_NO].name, TIMEOUT_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMDCOUNT_NO].name, CMDCOUNT_DEFAULT);  
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD1_ACT_NO].name, CMD1_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD1_NO].name, CMD1_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD1_CHECK_NO].name, CMD1_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD2_ACT_NO].name, CMD2_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD2_NO].name, CMD2_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD2_CHECK_NO].name, CMD2_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD3_ACT_NO].name, CMD3_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD3_NO].name, CMD3_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD3_CHECK_NO].name, CMD3_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD4_ACT_NO].name, CMD4_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD4_NO].name, CMD4_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD4_CHECK_NO].name, CMD4_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD5_ACT_NO].name, CMD5_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD5_NO].name, CMD5_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD5_CHECK_NO].name, CMD5_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD6_ACT_NO].name, CMD6_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD6_NO].name, CMD6_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD6_CHECK_NO].name, CMD6_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD7_ACT_NO].name, CMD7_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD7_NO].name, CMD7_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD7_CHECK_NO].name, CMD7_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD8_ACT_NO].name, CMD8_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD8_NO].name, CMD8_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD8_CHECK_NO].name, CMD8_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD9_ACT_NO].name, CMD9_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD9_NO].name, CMD9_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD9_CHECK_NO].name, CMD9_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD10_ACT_NO].name, CMD10_ACT_DEFAULT);
  fprintf (inifile, "%12s = \"%s\"\n", parmStruct[CMD10_NO].name, CMD10_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[CMD10_CHECK_NO].name, CMD10_CHECK_DEFAULT);
  fprintf (inifile, "%12s = %s\n", parmStruct[LANGUAGE_NO].name, LANGUAGE_DEFAULT);

  fclose (inifile);

  return OK;
}


/*! Liest ini-Datei und ueberprueft Syntax. Falls Datei ok,
    werden die Werte in die Struktur parmStruct eingetragen,
    falls nicht, wird die Datei kopiert und eine neue default-Datei
    geschrieben.
*/
int checkInifile (HWND hParent)
{
  FILE* inifile;
  char  filename[CHARSPERSTR] = INIFILENAME;
  char  datline[CHARSPERLINE];
  char  param[CHARSPERSTR] = "";
  char  value[CHARSPERSTR] = "";
  char  errStr[CHARSPERMESSAGESTR];
  int   endOfFile = FALSE;
  int   linecount = 0;
  int   checkDatline;
  extern parmStructT parmStruct[PARMCOUNT];


  inifile = fopen (filename, "r");

  if (inifile == NULL) {  // inifile kann nicht geoeffnet werden
    initAllValues (); // Default-Werte in parmStruct schreiben
    sprintf (errStr, "%s not found!\nWriting file with default values...", INIFILENAME);
    MessageBox (hParent, errStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
    if (writeDefaultInifile () == MY_ERROR) {
      MessageBox (hParent, "Cannot write ini-file!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
      return OK;
    }
    else return OK;
  }
  
  else {  // inifile geoeffnet
    // File zeilenweise auswerten, bis Zeilenende erreicht oder Fehler aufgetreten
    while (endOfFile == FALSE) {
      if (fgets (datline, CHARSPERLINE, inifile) != NULL) {
        linecount++;
        checkDatline = extractStringsDatline (datline, param, value);

        if (checkDatline == OK) {
          // Zeile ok, Parameter auswerten
          if (checkParamAndValue (param, value) == MY_ERROR) { // Parameter oder Value fehlerhaft, Meldung ausgeben
            fclose (inifile);
            sprintf (errStr, "Invalid parameter or value line %d in %s!\nWriting new file with default values...", linecount, INIFILENAME);
            MessageBox (hParent, errStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
            initAllValues (); // Default-Werte in parmStruct schreiben
            filecpy (INIFILENAME_COPY, INIFILENAME);
            if (writeDefaultInifile () == MY_ERROR) {
              MessageBox (hParent, "Cannot write ini-file!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
              return OK;
            }
            return OK;
          }
          else {} // Parameter und Value ok. Sie wurden in Fkt. checkParamAndValue ()
                  // in die Struktur parmStruct kopiert
        }

        // Zeile fehlerhaft, Meldung ausgeben
        else if (checkDatline == MY_ERROR) {
          fclose (inifile);
          sprintf (errStr, "Invalid line %d in %s!\nWriting new file with default values...", linecount, INIFILENAME);
          MessageBox (hParent, errStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
          initAllValues (); // Default-Werte in parmStruct schreiben
          filecpy (INIFILENAME_COPY, INIFILENAME);
          if (writeDefaultInifile () == MY_ERROR) {
            MessageBox (hParent, "Cannot write ini-file!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
            return OK;
          }
          return OK;
        }

        // Zeile leer oder nur Kommentar, nix tun
        else {}
              
      } // if (fgets...
    
      else {
        // Fileende erreicht
        endOfFile = TRUE;
      }
    }   // while
    fclose (inifile);
    
    // Pruefen, ob alle Parameter in parmStruct gefuellt sind
    // (Leerstring "" gilt auch als gefuellt)
    if (checkAllValuesFilled () == MY_ERROR) {
      sprintf (errStr, "Incomplete file %s!\nWriting new file with default values...", INIFILENAME);
      MessageBox (hParent, errStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
      initAllValues (); // Default-Werte in parmStruct schreiben
      filecpy (INIFILENAME_COPY, INIFILENAME);
      if (writeDefaultInifile () == MY_ERROR) {
        MessageBox (hParent, "Cannot write ini-file!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
        return OK;
      }
      return OK;
    }
    

    // Inifile komplett gelesen. Jetzt noch Wertebereiche einzelner Parameter checken.
    
    // Check, ob Server-IP-String und/oder Server-Name-String ok
    checkServer (hParent, parmStruct[SERVER_IP_NO].value, parmStruct[SERVER_NAME_NO].value);
    
    // Check, ob Timeout-Wert ok
    if (parmStruct[TIMEOUT_NO].valueInt < TIMEOUT_MIN || parmStruct[TIMEOUT_NO].valueInt > TIMEOUT_MAX) {
      sprintf (errStr, "Invalid Timeout value. Setting default value %s ms.", TIMEOUT_DEFAULT);
      MessageBox (hParent, errStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
      // Default-Wert eintragen
      strcpy (parmStruct[TIMEOUT_NO].value, TIMEOUT_DEFAULT);
      parmStruct[TIMEOUT_NO].valueInt = atoi (TIMEOUT_DEFAULT);
      // Inifile schreiben
      if (writeInifile () == MY_ERROR) {
        MessageBox (hParent, "Cannot write ini-file!", "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);
      }
    }
  }
  
  return OK; 
}


/*! Checkt, ob Server-IP-Adress-String leer ist und Server-Name-String nicht
    leer ist. Falls dies der Fall ist, versucht die Fkt. per myGethostbyname
    Name-String in eine IP aufzulosen. Rueckgabe OK, falls das gelingt, sonst
    MY_ERROR.
*/
int checkServer (HWND hWnd, char* ipStr, char* nameStr)
{
  int errNo;
  

  if (strcmp (ipStr, "") == 0) {
    // IP-String ist leer
    if (strcmp (nameStr, "") == 0) {
      // Name-String ist auch leer, nix zu tun
      return OK;
    }
    else {
      // Name-String gegeben, versuche Namen in IP aufzuloesen
      errNo = myGethostbyname (ipStr, nameStr);
      if (errNo != OK) {
        errMessageBox (hWnd, errNo, ipStr, nameStr);
        return MY_ERROR;
      }
      else {
        // IP ermittelt, trage IP-String in parmStruct ein
        strcpy (parmStruct[SERVER_IP_NO].value, ipStr);
        return OK;
      }
    }
  }
  else {
    // IP-String enthaelt gueltige IP
    return OK;
  }
  
   
  return MY_ERROR;  // darf nicht vorkommen
}


/*! Gibt Fehler gemaess Fehlernummer errNo in Messagebox aus
*/
void errMessageBox (HWND hWnd, int errNo, char* ipStr, char* nameStr)
{
  char errStr[CHARSPERMESSAGESTR];
  

  switch (errNo) {
    case ERR_NOIP: {
      sprintf (errStr, "Please specify server IP address or name in Menu 'File - Options'!");
      break;
    }

    case ERR_WSA: {
      sprintf (errStr, "Exception error: Cannot initialize winsock!");
      break;
    }

    case ERR_CMD: {
      sprintf (errStr, "Exception error: Unknown server command!");
      break;
    }

    case ERR_SOCK1: {
      sprintf (errStr, "Exception error: Cannot initialize socket!");
      break;
    }

    case ERR_SOCK2: {
      sprintf (errStr, "Exception error: WSAAsyncSelect failed!");
      break;
    }

    case ERR_CONNECT1: {
      sprintf (errStr, "Cannot connect server %s (Connect Error 1)!", ipStr);
      break;
    }

    case ERR_CONNECT2: {
      sprintf (errStr, "Cannot connect server %s (Connect Error 2)!", ipStr);
      break;
    }

    case ERR_SEND: {
      sprintf (errStr, "Error sending data to server %s!", ipStr);
      break;
    }
    
    case ERR_RECEIVE: {
      sprintf (errStr, "Error receiving data from server %s!", ipStr);
      break;
    }

   case ERR_RESOLVE_IP: { 
      sprintf (errStr, "Cannot resolve server name %s to IP address!\nTry connection with IP address.", nameStr);
      break;
    }
    
   case ERR_HELPFILE: { 
      strcpy (errStr, "Helpfile not found!");
      break;
    }

   case ERR_TIMEOUT: { 
      strcpy (errStr, "Timeout server command!");
      break;
    }

    default:
      sprintf (errStr, "Unknown Error!");
      break;
  }

  MessageBox (hWnd, errStr, "cnRemoteControl - Error", MB_OK|MB_ICONSTOP);

  return;
}


/*!
********************************************************************************
* MAIN                                                                         *
********************************************************************************
*/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, 
                    LPSTR lpCmdLine, int nCmdShow)
{
  WNDCLASS WndClass;
  MSG      msg;
  HACCEL   hAccel;
  extern HWND ghWnd;
  
   
  WndClass.style         = CS_HREDRAW | CS_VREDRAW;
  WndClass.cbClsExtra    = 0;
  WndClass.cbWndExtra    = DLGWINDOWEXTRA;
  WndClass.lpfnWndProc   = WndProc;
  WndClass.hInstance     = hInstance;
  WndClass.hbrBackground = (HBRUSH) COLOR_WINDOW;
  WndClass.hCursor       = LoadCursor (NULL, IDC_ARROW);
  WndClass.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_ICON));
  WndClass.lpszMenuName  = 0;
  WndClass.lpszClassName = "MAIN_WINDCLASS";
  
  if (!RegisterClass (&WndClass)) {
    return 0;
  }
  
  hInst = hInstance;
  ghWnd = CreateDialog (hInstance, MAKEINTRESOURCE(ID_MAINDLG), 0, NULL);  
    
  PostMessage (ghWnd, WM_COMMAND, IDC_INIT, 0); // Aufruf von Funktion init ()

  hAccel = LoadAccelerators (hInstance, "MAIN_WINDCLASS");
    
  // Main-Message-Schleife Hauptfenster, erweitert um if-Abfrage fuer
  // modusloses Dialogfenster als Mainfenster und Accelerators-Verarbeitung
  while (GetMessage (&msg, (HWND) NULL, 0, 0)) { 
    if (ghWnd == (HWND) NULL || !IsDialogMessage (ghWnd, &msg) && !TranslateAccelerator (ghWnd, hAccel, &msg)) { 
      TranslateMessage (&msg); 
      DispatchMessage (&msg);
    } 
  }
   
  return (int) (msg.wParam);
}
