/*
 *  cnRemoteControl - Remote control client application - see documentation
 *
 *  cnRemoteControl.h
 *
 *  Copyright (C) March 28, 2004 Christian Nause <cn@cntec.de>
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 */

#ifndef CNREMOTECONTROL_H
#define CNREMOTECONTROL_H


/*
********************************************************************************
* DEFINES                                                                      *
********************************************************************************
*/
#define BUILD_VERSION        "0.1.5"
#define BUILD_DATE           "28.03.2004"

#define TRUE                 1
#define FALSE                0

#define MY_ERROR             -1
#define OK                   0
#define EMPTY                1

#define TCPSENDBUFFERLEN     10
#define TCPRECVBUFFERLEN     2048
#define CHARSPERSTR          31
#define CHARSPERMESSAGESTR   256
#define CHARSPERLINE         101
#define CHARSPERBUTTONTEXT   20
#define INIFILENAME          "cnRemoteControl.ini"
#define INIFILENAME_COPY     "cnRemoteControl.in_"
#define HELPFILEPATH         "doc"
#define HELPFILENAME_GERMAN  "cnRemoteControl-de.html"
#define HELPFILENAME_ENGLISH "cnRemoteControl-en.html"

#define PARMCOUNT            36  // Anzahl der Parameter in parmStruct
#define BUTTONCOUNT          10  // Anzahl Buttons

#define SERVER_IP_NO         0
#define SERVER_NAME_NO       1
#define SERVER_PORT_NO       2
#define TIMEOUT_NO           3
#define CMDCOUNT_NO          4
#define CMD1_ACT_NO          5
#define CMD1_NO              6
#define CMD1_CHECK_NO        7
#define CMD2_ACT_NO          8
#define CMD2_NO              9
#define CMD2_CHECK_NO        10
#define CMD3_ACT_NO          11
#define CMD3_NO              12
#define CMD3_CHECK_NO        13
#define CMD4_ACT_NO          14
#define CMD4_NO              15
#define CMD4_CHECK_NO        16
#define CMD5_ACT_NO          17
#define CMD5_NO              18
#define CMD5_CHECK_NO        19
#define CMD6_ACT_NO          20
#define CMD6_NO              21
#define CMD6_CHECK_NO        22
#define CMD7_ACT_NO          23
#define CMD7_NO              24
#define CMD7_CHECK_NO        25
#define CMD8_ACT_NO          26
#define CMD8_NO              27
#define CMD8_CHECK_NO        28
#define CMD9_ACT_NO          29
#define CMD9_NO              30
#define CMD9_CHECK_NO        31
#define CMD10_ACT_NO         32
#define CMD10_NO             33
#define CMD10_CHECK_NO       34
#define LANGUAGE_NO          35

#define SERVER_IP_DEFAULT    "127.0.0.1"
#define SERVER_NAME_DEFAULT  "localhost"
#define SERVER_PORT_DEFAULT  "26056"
#define TIMEOUT_DEFAULT      "3000"
#define CMDCOUNT_DEFAULT     "10"
#define CMD1_DEFAULT         "Buttontext 1"
#define CMD1_CHECK_DEFAULT   "0"
#define CMD1_ACT_DEFAULT     "1"
#define CMD2_DEFAULT         "Buttontext 2"
#define CMD2_CHECK_DEFAULT   "0"
#define CMD2_ACT_DEFAULT     "1"
#define CMD3_DEFAULT         "Buttontext 3"
#define CMD3_CHECK_DEFAULT   "0"
#define CMD3_ACT_DEFAULT     "1"
#define CMD4_DEFAULT         "Buttontext 4"
#define CMD4_CHECK_DEFAULT   "0"
#define CMD4_ACT_DEFAULT     "1"
#define CMD5_DEFAULT         "Buttontext 5"
#define CMD5_CHECK_DEFAULT   "0"
#define CMD5_ACT_DEFAULT     "1"
#define CMD6_DEFAULT         "Buttontext 6"
#define CMD6_CHECK_DEFAULT   "0"
#define CMD6_ACT_DEFAULT     "1"
#define CMD7_DEFAULT         "Buttontext 7"
#define CMD7_CHECK_DEFAULT   "0"
#define CMD7_ACT_DEFAULT     "1"
#define CMD8_DEFAULT         "Buttontext 8"
#define CMD8_CHECK_DEFAULT   "0"
#define CMD8_ACT_DEFAULT     "1"
#define CMD9_DEFAULT         "Buttontext 9"
#define CMD9_CHECK_DEFAULT   "0"
#define CMD9_ACT_DEFAULT     "1"
#define CMD10_DEFAULT        "Buttontext 10"
#define CMD10_CHECK_DEFAULT  "0"
#define CMD10_ACT_DEFAULT    "1"
#define LANGUAGE_DEFAULT     "english"

#define TIMEOUT_MIN          0      // Minimum Timeout in milliseconds
#define TIMEOUT_MAX          20000  // Maximum Timeout in milliseconds

#define CMD01SENDSTRING      "Button01"
#define CMD02SENDSTRING      "Button02"
#define CMD03SENDSTRING      "Button03"
#define CMD04SENDSTRING      "Button04"
#define CMD05SENDSTRING      "Button05"
#define CMD06SENDSTRING      "Button06"
#define CMD07SENDSTRING      "Button07"
#define CMD08SENDSTRING      "Button08"
#define CMD09SENDSTRING      "Button09"
#define CMD10SENDSTRING      "Button10"

#define BOLV                 0  // Boolean-Value
#define INTV                 1  // Integer-Value der Form -123 oder +123 oder 123
#define FLOV                 2  // Float-Value der Form 1e-3 oder -1131e-23
#define HEXV                 3  // Hex-Value der Form 0x323 oder 0X323
#define STRV                 4  // String-Value
#define PORV                 5  // IP-Port-Value
#define IP4V                 6  // IP-Adresse (Ipv4-Format)

#define MENU_FILE_OPTIONS    11
#define MENU_FILE_EXIT       12
#define MENU_HELP_TOPICS     21
#define MENU_HELP_ABOUT      22

#define BUTTON1              51
#define BUTTON2              52
#define BUTTON3              53
#define BUTTON4              54
#define BUTTON5              55
#define BUTTON6              56
#define BUTTON7              57
#define BUTTON8              58
#define BUTTON9              59
#define BUTTON10             60

#define ERR_NOIP             10
#define ERR_WSA              11
#define ERR_CMD              12
#define ERR_SOCK1            13
#define ERR_SOCK2            14
#define ERR_CONNECT1         15
#define ERR_CONNECT2         16
#define ERR_SEND             17
#define ERR_RECEIVE          18
#define ERR_RESOLVE_IP       20
#define ERR_HELPFILE         21
#define ERR_TIMEOUT          22
#define ERR_UNKNOWN          99

#define WM_SOCKET_NOTIFY     0x0401
#define WM_SOCKET_CLOSE      0x0402
#define ID_TIMER             0x0403

typedef struct {
  char name[CHARSPERSTR];  // Parameter-Name
  int  number;             // Parameter-Nummer
  int  valueType;          // Value-Typen: BOLV, INTV, FLOV, HEXV, STRV, PORV oder IP4V
  char value[CHARSPERSTR]; // Parameter-Value-String (mit Vorbelegung)
  int  valueInt;           // Parameter-Integer falls valueType INTV oder HEXV
} parmStructT;


/*
********************************************************************************
* PROTOTYPES                                                                   *
********************************************************************************
*/
LRESULT CALLBACK OptionsMsgHandler (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
void init (HWND hWnd);
LRESULT CALLBACK WndProc (HWND hWnd, UINT uiMessage, WPARAM wParam, LPARAM lParam);
int writeInifile (void);
int writeDefaultInifile (void);
int checkInifile (HWND hParent);
int checkServer (HWND hWnd, char* ipStr, char* nameStr);
void errMessageBox (HWND hWnd, int errNo, char* ipStr, char* nameStr);


#endif
