/*
 *  cnRemoteControl - Remote control client application - see documentation
 *
 *  commControl.h
 *
 *  Copyright (C) March 22, 2004 Christian Nause <cn@cntec.de>
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 */
 
/*
********************************************************************************
* INCLUDES                                                                     *
********************************************************************************
*/
#include <stdio.h>
#include <sys/types.h>
#include <winsock.h>
#include "commControl.h"
#include "cnRemoteControl.h"


/*
********************************************************************************
* GLOBALS                                                                      *
********************************************************************************
*/
int gWSAStartupFlag = FALSE;


/*
********************************************************************************
* FUNCTIONS                                                                    *
********************************************************************************
*/
/*! IP-Adresse zu String nameStr ermitteln und als String in ipStr eintragen.
*/
int myGethostbyname (char* ipStr, char* nameStr)
{
  WSADATA wsa;
  struct hostent* pHostEnt;
  
  // Mauszeiger -> Sanduhr
  SetCursor (LoadCursor (NULL, IDC_WAIT));

  // Winsock initialisieren, falls nicht schon geschehen
  if (gWSAStartupFlag == FALSE) { 
    if (WSAStartup (MAKEWORD (1, 1), &wsa)) {
      return ERR_WSA;
    }
    else {
      gWSAStartupFlag = OK;
    }
  }

  pHostEnt = gethostbyname (nameStr);
  if (!pHostEnt) {
    return ERR_RESOLVE_IP;
  }
  else {
  	sprintf (ipStr, "%u.%u.%u.%u",
			(unsigned char) pHostEnt->h_addr_list[0][0],
			(unsigned char) pHostEnt->h_addr_list[0][1],
			(unsigned char) pHostEnt->h_addr_list[0][2],
			(unsigned char) pHostEnt->h_addr_list[0][3]);
	  return OK;
  }
  
  return ERR_UNKNOWN;
}


/*! Socket oeffnen und TCP/IP-Kommando zum Server schicken
*/
int openSocket (int port, char* ipStr, HWND hWnd)
{
  WSADATA wsa;
  extern SOCKET sock;
  struct sockaddr_in srv;
   

  // Abbruch, falls ipStr leer
  if (strcmp (ipStr, "") == 0) return ERR_NOIP;
  
  // Winsock initialisieren, falls nicht schon geschehen
  if (gWSAStartupFlag == FALSE) { 
    if (WSAStartup (MAKEWORD (1, 1), &wsa)) {
      return ERR_WSA;
    }
    else {
      gWSAStartupFlag = TRUE;
    }
  }

  // Mauszeiger -> Sanduhr
  SetCursor (LoadCursor (NULL, IDC_WAIT));

  // Socket initialisieren
  sock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (sock == INVALID_SOCKET) {
    WSACleanup ();
    // Mauszeiger -> Standardzeiger
    SetCursor (LoadCursor (NULL, IDC_ARROW));
    gWSAStartupFlag = FALSE;
    return ERR_SOCK1;
  }
  
  // Blockierende Socketfunktionen in einen eigenen Thread legen.
  // Diese Funktionen senden nach ihrer Beendigung WM_SOCKET_NOTIFY.
  if (WSAAsyncSelect (sock, hWnd, WM_SOCKET_NOTIFY, FD_CONNECT | FD_READ | FD_WRITE) == SOCKET_ERROR) {
    closesocket (sock);
    WSACleanup ();
    // Mauszeiger -> Standardzeiger
    SetCursor (LoadCursor (NULL, IDC_ARROW));
    gWSAStartupFlag = FALSE;
    return ERR_SOCK2;
  }  

  // IP-Addr., Port und Protokoll angeben
  srv.sin_addr.s_addr = inet_addr (ipStr);
  srv.sin_port = htons ((unsigned short int) port);
  srv.sin_family = AF_INET;

  // "connect" wird wegen dem vorherigen WSAAsyncSelect-Aufrufs SOCKET_ERROR liefern, 
  // da die Funktion als potentiell blockierende Funktion die Kontrolle unmittelbar 
  // (also noch vor dem Herstellen der Verbindung) zurueckgibt. Der Statuscode 
  // WSAEWOULDBLOCK wird deshalb akzeptiert und ist nicht als Fehler zu betrachten.
  connect (sock, (struct sockaddr*) &srv, sizeof (srv));

  if (WSAGetLastError () != WSAEWOULDBLOCK) {
    closesocket (sock) ;
    WSACleanup () ;
    // Mauszeiger -> Standardzeiger
    SetCursor (LoadCursor (NULL, IDC_ARROW));
    gWSAStartupFlag = FALSE;
    return ERR_CONNECT1;
  }
    
  return OK;	
}
