/*
 *  cnRemoteControl - Remote control client application - see documentation
 *
 *  parfile.c
 *
 *  Copyright (C) March 27, 2004 Christian Nause <cn@cntec.de>
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 */

/*
********************************************************************************
* INCLUDES                                                                     *
********************************************************************************
*/
#include <stdio.h>
#include <winsock.h>
#include "parfile.h"
#include "cnRemoteControl.h"


/*
********************************************************************************
* GLOBALS                                                                      *
********************************************************************************
*/


/*
********************************************************************************
* FUNCTIONS                                                                    *
********************************************************************************
*/
/* Zeichenumwandlung in einem String:
   Tabulator \t in zwei Spaces
   Linefeed \n in Carridge Return & Linefeed \r\n
   Carridge Return \r in Carridge Return & Linefeed \r\n

   Falls \r oder \n das vorletzte Zeichen von newStr ist (vor dem
   Endezeichen \0), wird \r\n an das Ende des Strings geschrieben,
   ohne dass der String unzulaessig verlaengert wird.
   
   Durch das Einfuegen von Zeichen wird newStr laenger, die maximale
   Laenge maxLength wird aber nicht ueberschritten.
*/
void filterString (char* newStr, char* oldStr, int maxLength)
{
  int i, j, oldLength;
  
  
  if ((int) strlen (oldStr) < maxLength) oldLength = (int) strlen (oldStr);
  else {
    oldLength = maxLength - 1;
    maxLength--; // Platz fuer \0 lassen
  }
    
  for (i = 0, j = 0; i < oldLength; i++, j++) {
    switch (oldStr[i]) {
      case '\t':
        newStr[j++] = ' ';
        newStr[j] = ' ';
      break;
      
      case '\r':
        if (oldStr[i+1] == '\n') break;
        else {
          newStr[j++] = '\r';
          newStr[j] = '\n';
        }
      break;
      
      case '\n':
        newStr[j++] = '\r';
        newStr[j] = '\n';
      break;
      
      default:
        newStr[j] = oldStr[i];
    }
    if (j >= maxLength) break;
  }
  

  newStr[j - 2] = '\r';
  newStr[j - 1] = '\n';
  newStr[j] = '\0';
  
  if (newStr[j - 3] == '\r') {
    newStr[j - 2] = '\n';
    newStr[j - 1] = '\0';
  }
  else if (newStr[j - 4] == '\r') {
    newStr[j - 3] = '\n';
    newStr[j - 2] = '\0';
  }
 
  return;
}


/*! Entfernen des Zeichens & aus String str
*/
void removeAmpersand (char *str)
{
  int i, pos;
  
  pos = -1;
  for (i = 0; i < (int) strlen (str); i++) {
    if (str[i] == '&') {
      pos = i;
      break;
    }
  }
  
  if (pos > -1)
    for (i = pos; i <= (int) strlen (str); i++)
      str[i] = str[i+1];
  
  return;
}


/*! Extrahiert Stings PARAM = VALUE aus ini-Datei-Zeile.
    Alle Zeilen einer ini-Datei der Form
      # Kommentar
      PARAM1=VALUE1  # Kommentar
      PARAM2:VALUE2  ; Kommentar
      PARAM3 = "VALUE3"
        ...
      PARAMN=VALUEN
    werden geparst. Checkt Syntax und liefert Zeiger auf String
    param und value und Integer OK, MY_ERROR oder EMPTY zurueck.
*/
int extractStringsDatline (char* line, char* param, char* value)
{
  int end = FALSE;
  int i = 0;
  int cpos = 0, bpos = 0, epos = 0, temp = 0;

  
  strcpy (param, "");
  strcpy (value, "");
    
  // Unterscheidung Kommentarzeile / Parameterzeile / Leerzeile
  while (end == FALSE) {
    // Vermutl. Parameterzeile
    if (isChar (line[i])) {
      line = &line[i];  // Fuehrende Whitespaces loeschen
      end = TRUE;
    }
    
    // Whitespace, weiter untersuchen
    else if (isSpace (line[i])) {
      end = FALSE;
      i++;
    }
    
    // Kommentar oder Zeilenende, Abbruch und Return mit EMPTY
    else if (isFinal (line[i]) || isComment (line[i])) {
      return EMPTY;
    }
      
    // Sonstiges Zeichen, Abbruch und Return mit MY_ERROR
    else return MY_ERROR;
        
    // Zeile zu lang, Abbruch und Return mit MY_ERROR
    if (i == CHARSPERLINE - 1) {
      return MY_ERROR;
    }
  }


  // Extraktion der Strings 'Parameter' und 'Value'
  // Trennzeichen = oder : finden
  for (i = 0; i < (int) strlen (line); i++) {
    if (isSeparator (line[i])) {
      cpos = i;
      break;
    }
    cpos = i;
  }
  if (cpos == 0 || cpos >= (int) strlen (line) || cpos >= CHARSPERLINE - 3) return MY_ERROR;
  
  // Erstes Zeichen des Parameter-Strings finden
  for (i = 0; i < cpos; i++) {
    if (isChar (line[i])) {
      bpos = i;
      break;
    }
    bpos = i;
  }
  if (bpos >= cpos) return MY_ERROR;

  // Letztes Zeichen des Parameter-Strings finden
  for (i = cpos - 1; i >= bpos; i--) {
    if (isChar (line[i])) {
      epos = i;
      break;
    }
  }
  if (epos >= CHARSPERLINE - 3) return MY_ERROR;

  // Parameter-String zusammensetzen
  if ((epos - bpos + 1) < CHARSPERSTR) {
    strncpy (param, &line[bpos], epos-bpos+1);
    param[epos-bpos+1] = '\0';
  }
  // String zu lang
  else return MY_ERROR;


  // Erstes Zeichen char oder Anfuehrungsstrich " des Value-Strings finden
  for (i = cpos + 1; i <= (int) strlen (line); i++) {
    if ((isChar (line[i])) || (isQuotation (line[i]))) {
      bpos = i;
      break;
    }
    if (isComment (line[i])) return MY_ERROR;
    if (isSeparator (line[i])) return MY_ERROR;
    bpos = i;
  }
  if (bpos >= (CHARSPERLINE - 3) || bpos >= (int) strlen (line)) return MY_ERROR;
  
  // Letztes Zeichen des Value-Strings finden
  // Falls Kommentar hinter Parameter steht, diesen abschneiden
  for (i = cpos + 1; i <= (int) strlen (line); i++) {
    if (isComment (line[i])) {
      temp = i - 1;
      break;
    }
  }
  
  // Falls kein Kommentar, beginne am Zeilenende zu suchen,
  // sonst beginne vor Kommentarzeichen zu suchen
  if (temp == 0) temp = (int) strlen (line);
  for (i = temp; i >= bpos; i--) {
    if ((isChar (line[i])) || (isQuotation (line[i]))) {
      epos = i;
      break;
    }
    epos = i;
  }
  if (epos >= CHARSPERLINE - 3) return MY_ERROR;

  // Fehler, falls value nur aus einem Anfuehrungszeichen besteht
  if ((bpos == epos) && (line[bpos] == '\"')) return MY_ERROR;
  
  // Value-String zusammensetzen
  if ((epos - bpos + 1) < CHARSPERSTR) {
    strncpy (value, &line[bpos], epos-bpos+1);
    value[epos-bpos+1] = '\0';
  }
  // String zu lang
  else return MY_ERROR;

  
  // Strings auf Sonderzeichen checken, diese sind nicht erlaubt
  // Ausnahmen siehe Fkt. isChar () und Space und Tabulator
  for (i = 0; i <= ((int) strlen (param) - 1); i++) {
    if (isChar (param[i]) == FALSE) {
      if (isSpace (param[i]) == FALSE) return MY_ERROR;
    }
  }
  
  // Falls Value-String in Anfuehrungsstrichen, diese nicht mitchecken
  if (value[0] == '\"') {
    if (value[(int) strlen (value) - 1] != '\"') {
      return MY_ERROR;
    }
    else {
      bpos = 1;
      epos = (int) strlen (value) - 2;
    }
  }
  else {
    bpos = 0;
    epos = (int) strlen (value) - 1;
  }
  
  for (i = bpos; i <= epos; i++) {
    if (isChar (value[i]) == FALSE) {
      if (isSpace (value[i]) == FALSE) return MY_ERROR;
    }
  }
   
  return OK;
}


/*! Checkt, ob der String value dem vorgegebenen Typ des Values
    in parmStruct entspricht
*/
int checkValueTyp (int parmNo, char* value)
{
  extern parmStructT parmStruct[PARMCOUNT];


  switch (parmStruct[parmNo].valueType) {
    case BOLV: return ascBoolCheck (value);
    
    case INTV: return ascIntCheck (value);
    
    case FLOV: return ascFloatCheck (value);
    
    case HEXV: return ascHexCheck (value);
    
    case STRV: return OK;
    
    case PORV: {
      if (ascIntCheck (value) == OK) {
        return ascPortCheck (value);
      }
      else return MY_ERROR;
    }

    case IP4V: {
      if (strcmp (value, "\"\"") == 0) {
        // IP-String leer
        return OK;
      }
      else {
        if (ascIPCheck (value) == OK) {
          // Gueltiger IP-String
          return OK;
        }
        else {
          // Ungueltiger IP-String
          return MY_ERROR;
        }
      }
    }
        
    default: return MY_ERROR;  // Darf nicht aufgerufen werden
  }
}


/*! Ist Parameter-String in Struktur parmStruct enthalten?
    Falls ja, Value auf richtigen Typ untersuchen und ueberpruefen,
    ob der Value-String bisher noch nicht eingetragen wurde.
    Wenn alles ok, Value in Struktur parmStruct kopieren.
*/
int checkParamAndValue (char* param, char* value)
{
  int i = 0;
  extern parmStructT parmStruct[PARMCOUNT];


  // Schleife ueber alle Eintraege in parmStruct
  while (i < PARMCOUNT) {
    if (strcmp (param, parmStruct[i].name) == 0) {
      // Parameter gefunden. Ist der Value vom richtigen Typ?
      // Wenn nicht, Abbruch mit Fehler
      if (checkValueTyp (parmStruct[i].number, value) == OK) {
        // Value ok. Aber: Steht in parmStruct der String "NA"?
        // Wenn nicht, kommt der Parameter mehrfach in der ini-
        // Datei vor. In diesem Fall Abbruch mit Fehler.
        if (strcmp (parmStruct[i].value, "NA") == 0) {
          // Alles ok, Value-String in parmStruct kopieren
          removeQuotation (value);  // Vorher ggf. " entfernen
          strncpy (parmStruct[i].value, value, CHARSPERSTR);
          if (parmStruct[i].valueType == INTV || parmStruct[i].valueType == HEXV) {
            // Falls valueType INTV oder HEXV den Eintrag 
            // parmStruct[i].value zusaetzlich mit Integer-Wert belegen
            parmStruct[i].valueInt = atoi (value);
          }
          if (parmStruct[i].valueType == BOLV) {
            // Falls valueType BOLV den Eintrag 
            // parmStruct[i].value zusaetzlich mit Integer-Wert 0 (FALSE) 
            // oder 1 (TRUE) belegen
            if (strcmp (parmStruct[i].value, "true") == 0 || 
                strcmp (parmStruct[i].value, "yes") == 0 ||
                strcmp (parmStruct[i].value, "1") == 0) 
              parmStruct[i].valueInt = TRUE;
            else parmStruct[i].valueInt = FALSE;
          }
          return OK;
        }
        else return MY_ERROR;
      }
      else return MY_ERROR;
    }
    // Weitersuchen, da Parameter bisher nicht in parmStruct
    else i++;
  }
  
  // parmStruct erfolglos durchsucht
  // String param ist ein unbekannter Parameter, Ende mit Fehler
  return MY_ERROR;
}


/*! Sind alle Parameter-Values von parmStruct gefuellt?,
    d.h. sind alle Value-String ungleich "NA"?
*/
int checkAllValuesFilled (void)
{
  int i;
  extern parmStructT parmStruct[PARMCOUNT];
  
  
  for (i = 0; i < PARMCOUNT; i++) {
    if (strcmp (parmStruct[i].value, "NA") == 0) return MY_ERROR;
  }
  
  return OK;
}


/*! Alle Parameter-Values mit Default-Strings vorbelegen
*/
void initAllValues (void)
{
  extern parmStructT parmStruct[PARMCOUNT];
  
  strcpy (parmStruct[SERVER_IP_NO].value,   SERVER_IP_DEFAULT);
  strcpy (parmStruct[SERVER_NAME_NO].value, SERVER_NAME_DEFAULT);
  strcpy (parmStruct[SERVER_PORT_NO].value, SERVER_PORT_DEFAULT);
  strcpy (parmStruct[TIMEOUT_NO].value,     TIMEOUT_DEFAULT);
  strcpy (parmStruct[CMDCOUNT_NO].value,    CMDCOUNT_DEFAULT);
  
  strcpy (parmStruct[CMD1_ACT_NO].value,    CMD1_ACT_DEFAULT);  
  strcpy (parmStruct[CMD1_NO].value,        CMD1_DEFAULT);
  strcpy (parmStruct[CMD1_CHECK_NO].value,  CMD1_CHECK_DEFAULT);
  
  strcpy (parmStruct[CMD2_ACT_NO].value,    CMD2_ACT_DEFAULT);  
  strcpy (parmStruct[CMD2_NO].value,        CMD2_DEFAULT);
  strcpy (parmStruct[CMD2_CHECK_NO].value,  CMD2_CHECK_DEFAULT);

  strcpy (parmStruct[CMD3_ACT_NO].value,    CMD3_ACT_DEFAULT);  
  strcpy (parmStruct[CMD3_NO].value,        CMD3_DEFAULT);
  strcpy (parmStruct[CMD3_CHECK_NO].value,  CMD3_CHECK_DEFAULT);

  strcpy (parmStruct[CMD4_ACT_NO].value,    CMD4_ACT_DEFAULT);  
  strcpy (parmStruct[CMD4_NO].value,        CMD4_DEFAULT);
  strcpy (parmStruct[CMD4_CHECK_NO].value,  CMD4_CHECK_DEFAULT);

  strcpy (parmStruct[CMD5_ACT_NO].value,    CMD5_ACT_DEFAULT);  
  strcpy (parmStruct[CMD5_NO].value,        CMD5_DEFAULT);
  strcpy (parmStruct[CMD5_CHECK_NO].value,  CMD5_CHECK_DEFAULT);

  strcpy (parmStruct[CMD6_ACT_NO].value,    CMD6_ACT_DEFAULT);  
  strcpy (parmStruct[CMD6_NO].value,        CMD6_DEFAULT);
  strcpy (parmStruct[CMD6_CHECK_NO].value,  CMD6_CHECK_DEFAULT);

  strcpy (parmStruct[CMD7_ACT_NO].value,    CMD7_ACT_DEFAULT);  
  strcpy (parmStruct[CMD7_NO].value,        CMD7_DEFAULT);
  strcpy (parmStruct[CMD7_CHECK_NO].value,  CMD7_CHECK_DEFAULT);

  strcpy (parmStruct[CMD8_ACT_NO].value,    CMD8_ACT_DEFAULT);  
  strcpy (parmStruct[CMD8_NO].value,        CMD8_DEFAULT);
  strcpy (parmStruct[CMD8_CHECK_NO].value,  CMD8_CHECK_DEFAULT);

  strcpy (parmStruct[CMD9_ACT_NO].value,    CMD9_ACT_DEFAULT);  
  strcpy (parmStruct[CMD9_NO].value,        CMD9_DEFAULT);
  strcpy (parmStruct[CMD9_CHECK_NO].value,  CMD9_CHECK_DEFAULT);

  strcpy (parmStruct[CMD10_ACT_NO].value,   CMD10_ACT_DEFAULT);  
  strcpy (parmStruct[CMD10_NO].value,       CMD10_DEFAULT);
  strcpy (parmStruct[CMD10_CHECK_NO].value, CMD10_CHECK_DEFAULT);

  return;
}


/*! Alle Grossbuchstaben des Strings str in Kleinbuchstaben wandeln
*/
void strtolower (char *str)
{
  int i;
  
  for (i = 0; i < (int) strlen (str); i++) {
    *(str+i) = tolower (*(str+i));
  }
  
  return;
}


/*! Alle Kleinbuchstaben des Strings in Grossbuchstaben wandeln
*/
void strtoupper (char *str)
{
  int i;
  
  for (i = 0; i < (int) strlen (str); i++) {
    *(str+i) = toupper (*(str+i));
  }
  
  return;
}


/*! Checkt String str, ob er ein gueltiger Boolean-Value 
    (0, 1, no, yes, false, true) ist.
    Rueckgabe MY_ERROR (=-1) im Fehlerfall, sonst OK (0).
*/
int ascBoolCheck (char* str)
{
  strtolower (str);
  removeQuotation (str);

  if (strcmp (str, "0") == 0)          return OK;
  else if (strcmp (str, "1") == 0)     return OK;
  else if (strcmp (str, "yes") == 0)   return OK;
  else if (strcmp (str, "no") == 0)    return OK;
  else if (strcmp (str, "true") == 0)  return OK;
  else if (strcmp (str, "false") == 0) return OK;
  
  return MY_ERROR;
}


/*! Checkt String str der Form 12345678 oder +12345678 oder -12345678,
    ob er eine gueltige Integer-Zahl ist.
    Rueckgabe MY_ERROR (=-1) im Fehlerfall, sonst OK (0).
*/
int ascIntCheck (char* str)
{
  char *pStr = str;
  int strSize = 0;
  int maxStrLength = 8;
  

  if ((*pStr == '+') || (*pStr == '-')) {
    maxStrLength = 9;
    pStr++;
  }
  
  while (*pStr != '\0') {
    if (isdigit (*pStr) == 0) { // Fehler, falls keine Ziffer
      return MY_ERROR;
    } else { }
    strSize++;
    
    if (strSize > 8) {          // Fehler, falls String zu lang
      return MY_ERROR;
    } else { }
    pStr++;
  }

  return OK;
}


/*! Checkt String str der Form +1.2345678e+12 (auch ohne +) oder
    -1.2E01 ob er eine gueltige Float-Zahl ist.
    Rueckgabe MY_ERROR (=-1) im Fehlerfall, sonst OK (0).
*/
int ascFloatCheck (char* str)
{
  char inStr[20];
  char *pStr;
  char *tok[3];
  int  strSize;
  
  
  strcpy (inStr, str);
  tok[0] = strtok (inStr, ".");
  tok[1] = strtok (NULL, "eE");
  tok[2] = strtok (NULL, "eE");
  
  if ((tok[0] == NULL) || (tok[1] == NULL) || (tok[2] == NULL)) return MY_ERROR;
  
  // Checke Mantisse (+m oder -m oder m)
  pStr = tok[0];
  if ((*pStr == '+') || (*pStr == '-')) {
    pStr++;
  }
  strSize = 0;
  while (*pStr != '\0') {
    if (isdigit (*pStr) == 0) { // Fehler, falls keine Ziffer
      return MY_ERROR;
    }
    strSize++;
    if (strSize > 1) {          // Fehler, falls String zu lang
      return MY_ERROR;
    }
    pStr++;
  }
  
  // Checke Dezimalteil (max. 7 Ziffern)
  pStr = tok[1];
  strSize = 0;
  while (*pStr != '\0') {
    if (isdigit (*pStr) == 0) { // Fehler, falls keine Ziffer
      return MY_ERROR;
    }
    strSize++;
    if (strSize > 7) {          // Fehler, falls String zu lang
      return MY_ERROR;
    }
    pStr++;
  }
  
  // Checke Exponent (-ee oder +ee oder ee)
  pStr = tok[2];
  if ((*pStr == '+') || (*pStr == '-')) {
    pStr++;
  }
  strSize = 0;
  while (*pStr != '\0') {
    if (isdigit (*pStr) == 0) { // Fehler, falls keine Ziffer
      return MY_ERROR;
    }
    strSize++;
    if (strSize > 2) {          // Fehler, falls String zu lang
      return MY_ERROR;
    }
    pStr++;
  }

  return OK;
}


/*! Checkt String str der Form 0x1234ABCD, ob er eine gueltige Hex-Zahl ist.
    Rueckgabe MY_ERROR (=-1) im Fehlerfall, sonst OK (0).
*/
int ascHexCheck (char* str)
{
  char* pStr = str;
  int   strSize = 0;
  int   maxStrLength = 8;


  if (*pStr != '0') {                     // Fehler, falls 1. Zeichen nicht 0
    return MY_ERROR;
  }
  pStr++;
  if ((*pStr != 'x') && (*pStr != 'X')) { // Fehler, falls 2. Zeichen nicht x oder X
    return MY_ERROR;
  }
  pStr++;
  while (*pStr != '\0') {
    if (isxdigit (*pStr) == 0) {          // Fehler, falls keine Hexziffer
      return MY_ERROR;
    }
    strSize++;
    if (strSize > maxStrLength) {         // Fehler, falls String zu lang
      return MY_ERROR;
    }
    pStr++;
  }

  return OK;
}


/*! Checkt String, ob er gueltige IP-Adresse enthaelt.
*/
int ascIPCheck (char* str)
{
  struct in_addr temp_addr;


  if (inet_addr (str) != INADDR_NONE) {
    // Umwandlung String in numerische IP und anschliessend
    // wieder zurueck in String.
    // So werden auch gueltige Adressen ausgewertet, die nicht in der
    // dotted-Schreibweise vorliegen
    temp_addr.s_addr = inet_addr (str);
    strcpy (str, inet_ntoa (temp_addr));
    return OK;
  }
  
  else return MY_ERROR;
}


/*! Checkt String, ob er ein Integer zwischen 1 und 65535 ist.
    Vor Aufruf sollte mit ascIntCheck (str) auf gueltigen
    Integer-String geprueft werden.
*/
int ascPortCheck (char* str)
{
  int c;
  
  c = atoi (str);  
  if ((c >=1) && (c <= 65535)) return OK;
  else return MY_ERROR;
}


/*! Checkt String, ob er nur gueltige Zeichen enthaelt.
    Siehe isChar und zusaetzlich Space und Leerzeichen
*/
int ascStringCheck (char* str)
{
  int i;
  
  for (i = 0; i < (int) strlen (str); i++) {
    if (isChar (str[i]) == FALSE) {
      if (isSpace (str[i]) == FALSE) return FALSE;
    }
  }
  
  return TRUE;
}


/*! Erkennung zulaessige Zeichen fuer Parameter- und
    Value-Strings
*/
int isChar (char c)
{
  if ((c >= 0x61) && (c <= 0x7A)) return TRUE;    // a...z
  if ((c >= 0x41) && (c <= 0x5A)) return TRUE;    // A...Z
  if ((c >= 0x30) && (c <= 0x39)) return TRUE;    // 0...9
  if ((c == '.') || (c == '\\') || (c == '/')) return TRUE;  
  if ((c == '-') || (c == '_') || (c == ''))  return TRUE;
  if ((c == '') || (c == '') || (c == ''))  return TRUE;
  if ((c == '') || (c == '') || (c == ''))  return TRUE;
  if (c == '&') return TRUE;
  return FALSE;
}


/*! Erkennung Leerzeichen und Tabulator
*/
int isSpace (char c)
{
  if ((c == ' ') || (c == 0x09)) return TRUE;
  return FALSE;
}


/*! Erkennung Kommentarzeichen # und ;
*/
int isComment (char c)
{
  if ((c == '#') || (c == ';')) return TRUE;
  return FALSE;
}


/*! Erkennung Trennzeichen = und :
*/
int isSeparator (char c)
{
  if ((c == '=') || (c == ':')) return TRUE;
  return FALSE;
}


/*! Erkennung Zeilenende (Carridge Return, Line-Feed und Stringendezeichen)
*/
int isFinal (char c)
{
  if ((c == 0x0D) || (c == 0x0A) || (c == '\0')) return TRUE;
  return FALSE;
}


/*! Erkennung Anfuehrungsstrich "
*/
int isQuotation (char c)
{
  if (c == '\"') return TRUE;
  return FALSE;
}


/*! Erkennung geoeffnete eckige Klammer [ fuer Section-Beginn
*/
int isSectionBegin (char c)
{
  if (c == '[') return TRUE;
  return FALSE;
}


/*! Erkennung geschlossene eckige Klammer ] fuer Section-Ende
*/
int isSectionEnd (char c)
{
  if (c == ']') return TRUE;
  return FALSE;
}


/*! Entfernt die Anfuehrungszeichen am Beginn und Ende eines Strings
*/
int removeQuotation (char* str)
{
  int i;
  
  
  // Abbruch, falls keine Anfuehrungszeichen vorhanden
  if ((str[0] != '\"') || (str[(int) strlen (str) - 1] != '\"')) return OK;
  
  for (i = 0; i < (int) strlen (str); i++) {
    str[i] = str[i+1];
  }
  str[i-2] = '\0';
  
  return OK;
}


/*! File-Copy: Legt binaere Kopie targetName des Files sourceName an
*/
int filecpy (char* targetName, char* sourceName)
{
  FILE *fsource, *ftarget;
  int c;
  
  
  if ((fsource = fopen (sourceName, "rb")) == NULL ) return MY_ERROR; 
  if ((ftarget = fopen (targetName, "wb")) == NULL ) return MY_ERROR;
  
  while (TRUE) {
    c = fgetc (fsource);
    if (!feof (fsource)) fputc (c, ftarget);
    else break;
  }
  
  fclose (fsource);
  fclose (ftarget);
  
  return OK;
}
